import { BehaviorSubject, Observable } from 'rxjs';
/**
 * A unified interface which various collaborative editing approaches
 * (CRDT with vector clock, CRDT with server clock, OT, client-only OT,
 * simple patch, etc.) need to implement.
 */
export interface BlockModel<Data, Patch> {
    /** Unique ID of this block. */
    /** Get latest data of this block. */
    getData(): Data;
    /** Create an independent copy of this block. */
    fork(): BlockModel<Data, Patch>;
    /** Apply a single patch of changes. */
    apply(patch: Patch): void;
}
export declare class BasicBlock<Data, Patch> {
    readonly model: BlockModel<Data, Patch>;
    v$: BehaviorSubject<number>;
    constructor(version: number, model: BlockModel<Data, Patch>);
    fork(): BasicBlock<Data, Patch>;
    /**
     * Observable of the latest data value of this block. It is wrapped in
     * a function to allow for lazy evaluation.
     */
    data$(): Observable<Data>;
    /** Get the latest value of the block. */
    getData(): Data;
    apply(patch: Patch): void;
}
export type PatchResponse<Patch> = true | false | Patch;
export interface BranchMergeResponse<Patch> {
    version: number;
    batches: PatchResponse<Patch>[][];
}
export declare class Branch<Data, Patch> {
    /** The last known confirmed value on the server. */
    protected readonly base$: BehaviorSubject<BasicBlock<Data, Patch>>;
    /** Block value including all latest changes optimistically applied on the client. */
    protected readonly head$: BehaviorSubject<BasicBlock<Data, Patch>>;
    /**
     * List of patches applied locally to the head, but not yet
     * added to the `batches` list.
     */
    protected patches: Patch[];
    /** List of batches to be persisted on the server. */
    protected batches: Patch[][];
    constructor(base: BasicBlock<Data, Patch>);
    cutBatch(): void;
    /** Apply a patch locally to the head. */
    commit(patch: Patch): void;
    /**
     * Advance the base to the latest known value from the server and
     * re-apply all local patches, after optional transformation.
     */
    rebase(patches: Patch[]): void;
    /** Set new base and reset all local changes. Returns local changes. */
    reset(base: BasicBlock<Data, Patch>): Patch[];
}
