"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Block = void 0;
const printTree_1 = require("tree-dump/lib/printTree");
const json_hash_1 = require("../../../json-hash");
const MarkerOverlayPoint_1 = require("../overlay/MarkerOverlayPoint");
const iterator_1 = require("../../../util/iterator");
const Inline_1 = require("./Inline");
class Block {
    constructor(txt, path, marker) {
        this.txt = txt;
        this.path = path;
        this.marker = marker;
        this.parent = null;
        this.children = [];
        // ----------------------------------------------------------------- Stateful
        this.hash = 0;
    }
    /**
     * @returns Stable unique identifier within a list of blocks. Used for React
     * or other rendering library keys.
     */
    key() {
        if (!this.marker)
            return this.tag();
        const id = this.marker.id;
        return id.sid.toString(36) + id.time.toString(36);
    }
    tag() {
        const path = this.path;
        const length = path.length;
        return length ? path[length - 1] : '';
    }
    attr() {
        return this.marker?.data();
    }
    /**
     * Iterate through all overlay points of this block, until the next marker
     * (regardless if that marker is a child or not).
     */
    points0(withMarker = false) {
        const txt = this.txt;
        const overlay = txt.overlay;
        const iterator = overlay.points0(this.marker);
        let closed = false;
        return () => {
            if (withMarker) {
                withMarker = false;
                return this.marker ?? overlay.START;
            }
            if (closed)
                return;
            const point = iterator();
            if (!point)
                return;
            if (point instanceof MarkerOverlayPoint_1.MarkerOverlayPoint) {
                closed = true;
                return;
            }
            return point;
        };
    }
    points(withMarker) {
        return new iterator_1.UndefEndIter(this.points0(withMarker));
    }
    tuples0() {
        const overlay = this.txt.overlay;
        const iterator = overlay.tuples0(this.marker);
        let closed = false;
        return () => {
            if (closed)
                return;
            const pair = iterator();
            if (!pair)
                return;
            if (!pair[1] || pair[1] instanceof MarkerOverlayPoint_1.MarkerOverlayPoint)
                closed = true;
            return pair;
        };
    }
    tuples() {
        return new iterator_1.UndefEndIter(this.tuples0());
    }
    texts0() {
        const txt = this.txt;
        const iterator = this.tuples0();
        return () => {
            const pair = iterator();
            return pair && Inline_1.Inline.create(txt, pair[0], pair[1]);
        };
    }
    texts() {
        return new iterator_1.UndefEndIter(this.texts0());
    }
    text() {
        let str = '';
        const iterator = this.texts0();
        let text = iterator();
        while (text) {
            str += text.text();
            text = iterator();
        }
        return str;
    }
    refresh() {
        const { path, children } = this;
        let state = 5381 /* CONST.START_STATE */;
        state = (0, json_hash_1.updateJson)(state, path);
        const marker = this.marker;
        if (marker) {
            state = (0, json_hash_1.updateNum)(state, marker.marker.refresh());
            state = (0, json_hash_1.updateNum)(state, marker.textHash);
        }
        else {
            state = (0, json_hash_1.updateNum)(state, this.txt.overlay.leadingTextHash);
        }
        for (let i = 0; i < children.length; i++)
            state = (0, json_hash_1.updateNum)(state, children[i].refresh());
        return (this.hash = state);
    }
    // ---------------------------------------------------------------- Printable
    toStringHeader() {
        const hash = `#${this.hash.toString(36).slice(-4)}`;
        const tag = `<${this.path.join('.')}>`;
        const header = `${this.constructor.name} ${hash} ${tag}`;
        return header;
    }
    toString(tab = '') {
        const header = this.toStringHeader();
        const hasChildren = !!this.children.length;
        return (header +
            (0, printTree_1.printTree)(tab, [
                this.marker ? (tab) => this.marker.toString(tab) : null,
                this.marker && hasChildren ? () => '' : null,
                hasChildren
                    ? (tab) => 'children' +
                        (0, printTree_1.printTree)(tab, this.children.map((child, i) => (tab) => `${i + 1}. ` + child.toString(tab + '  ' + ' '.repeat(String(i + 1).length))))
                    : null,
            ]));
    }
}
exports.Block = Block;
