"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Inline = void 0;
const printTree_1 = require("tree-dump/lib/printTree");
const stringify_1 = require("../../../json-text/stringify");
const Range_1 = require("../rga/Range");
const ChunkSlice_1 = require("../util/ChunkSlice");
const json_hash_1 = require("../../../json-hash");
/**
 * The `Inline` class represents a range of inline text within a block, which
 * has the same annotations and formatting for all of its text contents, i.e.
 * its text contents can be rendered as a single (`<span>`) element. However,
 * the text contents might still be composed of multiple {@link ChunkSlice}s,
 * which are the smallest units of text and need to be concatenated to get the
 * full text content of the inline.
 */
class Inline extends Range_1.Range {
    static create(txt, start, end) {
        const texts = [];
        txt.overlay.chunkSlices0(undefined, start, end, (chunk, off, len) => {
            if (txt.overlay.isMarker(chunk.id))
                return;
            texts.push(new ChunkSlice_1.ChunkSlice(chunk, off, len));
        });
        return new Inline(txt.str, start, end, texts);
    }
    constructor(rga, start, end, 
    /**
     * @todo PERF: for performance reasons, we should consider not passing in
     * this array. Maybe pass in just the initial chunk and the offset. However,
     * maybe even the just is not necessary, as the `.start` point should have
     * its chunk cached, or will have it cached after the first access.
     */
    texts) {
        super(rga, start, end);
        this.start = start;
        this.end = end;
        this.texts = texts;
    }
    /**
     * @returns A stable unique identifier of this *inline* within a list of other
     *     inlines of the parent block. Can be used for UI libraries to track the
     *     identity of the inline across renders.
     */
    key() {
        return (0, json_hash_1.updateNum)(this.start.refresh(), this.end.refresh());
    }
    /**
     * @returns The position of the inline withing the text.
     */
    pos() {
        const chunkSlice = this.texts[0];
        if (!chunkSlice)
            return -1;
        const chunk = chunkSlice.chunk;
        const pos = this.rga.pos(chunk);
        return pos + chunkSlice.off;
    }
    /**
     * @returns Returns the attributes of the inline, which are the slice
     *     annotations and formatting applied to the inline.
     */
    attr() {
        const attr = {};
        const point = this.start;
        const slices = this.texts.length ? point.layers : point.markers;
        const length = slices.length;
        for (let i = 0; i < length; i++) {
            const slice = slices[i];
            const type = slice.type;
            switch (slice.behavior) {
                case 4 /* SliceBehavior.Cursor */:
                case 1 /* SliceBehavior.Stack */: {
                    let dataList = attr[type] || (attr[type] = []);
                    if (!Array.isArray(dataList))
                        dataList = attr[type] = [dataList];
                    let data = slice.data();
                    if (data === undefined)
                        data = 1;
                    dataList.push(data);
                    break;
                }
                case 2 /* SliceBehavior.Overwrite */: {
                    let data = slice.data();
                    if (data === undefined)
                        data = 1;
                    attr[type] = data;
                    break;
                }
                case 3 /* SliceBehavior.Erase */: {
                    delete attr[type];
                    break;
                }
            }
        }
        // TODO: Iterate over the markers...
        return attr;
    }
    // ---------------------------------------------------------------- Printable
    toString(tab = '') {
        const str = this.text();
        const truncate = str.length > 32;
        const text = JSON.stringify(truncate ? str.slice(0, 32) : str) + (truncate ? ' …' : '');
        const startFormatted = this.start.toString(tab, true);
        const range = this.start.cmp(this.end) === 0 ? startFormatted : `${startFormatted} ↔ ${this.end.toString(tab, true)}`;
        const header = `${this.constructor.name} ${range} ${text}`;
        const marks = this.attr();
        const markKeys = Object.keys(marks);
        return (header +
            (0, printTree_1.printTree)(tab, [
                !markKeys.length
                    ? null
                    : (tab) => 'attributes' +
                        (0, printTree_1.printTree)(tab, markKeys.map((key) => () => key + ' = ' + (0, stringify_1.stringify)(marks[key]))),
                !this.texts.length
                    ? null
                    : (tab) => 'texts' +
                        (0, printTree_1.printTree)(tab, this.texts.map((text) => (tab) => text.toString(tab))),
            ]));
    }
}
exports.Inline = Inline;
