"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cursor = void 0;
const json_crdt_patch_1 = require("../../../json-crdt-patch");
const PersistedSlice_1 = require("../slice/PersistedSlice");
class Cursor extends PersistedSlice_1.PersistedSlice {
    get anchorSide() {
        return this.type;
    }
    // ---------------------------------------------------------------- mutations
    set anchorSide(value) {
        this.update({ type: value });
    }
    anchor() {
        return this.anchorSide === 0 /* CursorAnchor.Start */ ? this.start : this.end;
    }
    focus() {
        return this.anchorSide === 0 /* CursorAnchor.Start */ ? this.end : this.start;
    }
    set(start, end = start, anchorSide = this.anchorSide) {
        this.start = start;
        this.end = end === start ? end.clone() : end;
        this.update({
            range: this,
            type: anchorSide,
        });
    }
    /**
     * Move one of the edges of the cursor to a new point.
     *
     * @param point Point to set the edge to.
     * @param endpoint 0 for "focus", 1 for "anchor."
     */
    setEndpoint(point, endpoint = 0) {
        if (this.start === this.end)
            this.end = this.end.clone();
        let anchor = this.anchor();
        let focus = this.focus();
        if (endpoint === 0)
            focus = point;
        else
            anchor = point;
        if (focus.cmpSpatial(anchor) < 0)
            this.set(focus, anchor, 1 /* CursorAnchor.End */);
        else
            this.set(anchor, focus, 0 /* CursorAnchor.Start */);
    }
    move(move) {
        const { start, end } = this;
        start.move(move);
        if (start !== end)
            end.move(move);
        this.set(start, end);
    }
    /**
     * Ensures there is no range selection. If user has selected a range,
     * the contents is removed and the cursor is set at the start of the range as cursor.
     *
     * @todo If block boundaries are withing the range, remove the blocks.
     * @todo Stress test this method.
     *
     * @returns Returns the cursor position after the operation.
     */
    collapse() {
        const isCaret = this.isCollapsed();
        if (!isCaret) {
            const { start, end } = this;
            const delStartId = start.isAbsStart()
                ? this.txt.point().refStart().id
                : start.anchor === 0 /* Anchor.Before */
                    ? start.id
                    : start.nextId();
            const delEndId = end.isAbsEnd()
                ? this.txt.point().refEnd().id
                : end.anchor === 1 /* Anchor.After */
                    ? end.id
                    : end.prevId();
            if (!delStartId || !delEndId)
                throw new Error('INVALID_RANGE');
            const rga = this.rga;
            const spans = rga.findInterval2(delStartId, delEndId);
            const api = this.txt.model.api;
            api.builder.del(rga.id, spans);
            api.apply();
            if (start.anchor === 1 /* Anchor.After */)
                this.setAfter(start.id);
            else
                this.setAfter(start.prevId() || rga.id);
        }
    }
    /**
     * Insert inline text at current cursor position. If cursor selects a range,
     * the range is removed and the text is inserted at the start of the range.
     */
    insert(text) {
        if (!text)
            return;
        this.collapse();
        const after = this.start.clone();
        after.refAfter();
        const textId = this.txt.ins(after.id, text);
        const shift = text.length - 1;
        this.setAfter(shift ? (0, json_crdt_patch_1.tick)(textId, shift) : textId);
    }
    delBwd() {
        const isCollapsed = this.isCollapsed();
        if (isCollapsed) {
            const range = this.txt.findCharBefore(this.start);
            if (!range)
                return;
            this.set(range.start, range.end);
        }
        this.collapse();
    }
    // ---------------------------------------------------------------- Printable
    toStringName() {
        const focusIcon = this.anchorSide === 0 /* CursorAnchor.Start */ ? '.→|' : '|←.';
        return `${super.toStringName()}, ${focusIcon}`;
    }
}
exports.Cursor = Cursor;
