"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Editor = void 0;
const Cursor_1 = require("./Cursor");
const EditorSlices_1 = require("./EditorSlices");
class Editor {
    constructor(txt) {
        this.txt = txt;
        this.saved = new EditorSlices_1.EditorSlices(txt, txt.savedSlices);
        this.extra = new EditorSlices_1.EditorSlices(txt, txt.extraSlices);
        this.local = new EditorSlices_1.EditorSlices(txt, txt.localSlices);
    }
    // ------------------------------------------------------------------ cursors
    firstCursor() {
        const iterator = this.txt.localSlices.iterator0();
        let cursor = iterator();
        while (cursor) {
            if (cursor instanceof Cursor_1.Cursor)
                return cursor;
            cursor = iterator();
        }
        return;
    }
    /**
     * Returns the first cursor in the text. If there is no cursor, creates one
     * and inserts it at the start of the text. To work with multiple cursors, use
     * `.cursors()` method.
     *
     * Cursor is the the current user selection. It can be a caret or a range. If
     * range is collapsed to a single point, it is a *caret*.
     */
    get cursor() {
        const maybeCursor = this.firstCursor();
        if (maybeCursor)
            return maybeCursor;
        const txt = this.txt;
        const cursor = txt.localSlices.ins(txt.rangeAt(0), 4 /* SliceBehavior.Cursor */, 0 /* CursorAnchor.Start */, undefined, Cursor_1.Cursor);
        return cursor;
    }
    cursors(callback) {
        this.txt.localSlices.forEach((slice) => {
            if (slice instanceof Cursor_1.Cursor)
                callback(slice);
        });
    }
    delCursors() {
        this.cursors((cursor) => this.local.del(cursor));
    }
    // ------------------------------------------------------------- text editing
    /**
     * Insert inline text at current cursor position. If cursor selects a range,
     * the range is removed and the text is inserted at the start of the range.
     */
    insert(text) {
        this.cursors((cursor) => cursor.insert(text));
    }
    /**
     * Deletes the previous character at current cursor position. If cursor
     * selects a range, deletes the whole range.
     */
    delBwd() {
        this.cursors((cursor) => cursor.delBwd());
    }
    // ------------------------------------------------------------------ various
    /** @todo Add main impl details of this to `Cursor`, but here ensure there is only one cursor. */
    selectAll() {
        const range = this.txt.rangeAll();
        if (!range)
            return false;
        this.cursor.setRange(range);
        return true;
    }
    /** @deprecated use `.saved.insStack` */
    insStackSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 1 /* SliceBehavior.Stack */, type, data);
    }
    /** @deprecated use `.saved.insOverwrite` */
    insOverwriteSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 2 /* SliceBehavior.Overwrite */, type, data);
    }
    /** @deprecated use `.saved.insErase` */
    insEraseSlice(type, data) {
        const range = this.cursor.range();
        return this.txt.savedSlices.ins(range, 3 /* SliceBehavior.Erase */, type, data);
    }
    /** @deprecated use `.saved.insMarker` */
    insMarker(type, data) {
        return this.saved.insMarker(type, data, "\n" /* Chars.BlockSplitSentinel */)[0];
    }
}
exports.Editor = Editor;
