import { Point } from '../rga/Point';
import { OverlayPoint } from './OverlayPoint';
import { MarkerOverlayPoint } from './MarkerOverlayPoint';
import { ITimestampStruct } from '../../../json-crdt-patch/clock';
import { Range } from '../rga/Range';
import { UndefEndIter, UndefIterator } from '../../../util/iterator';
import type { Chunk } from '../../../json-crdt/nodes/rga';
import type { Peritext } from '../Peritext';
import type { Stateful } from '../types';
import type { Printable } from 'tree-dump/lib/types';
import type { Slice } from '../slice/types';
import type { OverlayPair, OverlayTuple } from './types';
/**
 * Overlay is a tree structure that represents all the intersections of slices
 * in the text. It is used to quickly find all the slices that overlap a
 * given point in the text. The overlay is a read-only structure, its state
 * is changed only by calling the `refresh` method, which updates the overlay
 * based on the current state of the text and slices.
 */
export declare class Overlay<T = string> implements Printable, Stateful {
    protected readonly txt: Peritext<T>;
    root: OverlayPoint<T> | undefined;
    root2: MarkerOverlayPoint<T> | undefined;
    /** A virtual absolute start point, used when the absolute start is missing. */
    readonly START: OverlayPoint<T>;
    /** A virtual absolute end point, used when the absolute end is missing. */
    readonly END: OverlayPoint<T>;
    constructor(txt: Peritext<T>);
    private point;
    private mPoint;
    first(): OverlayPoint<T> | undefined;
    last(): OverlayPoint<T> | undefined;
    /**
     * Retrieve overlay point or the previous one, measured in spacial dimension.
     */
    getOrNextLower(point: Point<T>): OverlayPoint<T> | undefined;
    /**
     * Retrieve overlay point or the next one, measured in spacial dimension.
     */
    getOrNextHigher(point: Point<T>): OverlayPoint<T> | undefined;
    /** @todo Rename to `chunks()`. */
    chunkSlices0(chunk: Chunk<T> | undefined, p1: Point<T>, p2: Point<T>, callback: (chunk: Chunk<T>, off: number, len: number) => void): Chunk<T> | undefined;
    points0(after: undefined | OverlayPoint<T>): UndefIterator<OverlayPoint<T>>;
    points(after?: undefined | OverlayPoint<T>): IterableIterator<OverlayPoint<T>>;
    markers0(after: undefined | MarkerOverlayPoint<T>): UndefIterator<MarkerOverlayPoint<T>>;
    markers(): UndefEndIter<MarkerOverlayPoint<T>>;
    pairs0(after: undefined | OverlayPoint<T>): UndefIterator<OverlayPair<T>>;
    pairs(after?: undefined | OverlayPoint<T>): IterableIterator<OverlayPair<T>>;
    tuples0(after: undefined | OverlayPoint<T>): UndefIterator<OverlayTuple<T>>;
    tuples(after?: undefined | OverlayPoint<T>): IterableIterator<OverlayTuple<T>>;
    /**
     * Finds the first point that satisfies the given predicate function.
     *
     * @param predicate Predicate function to find the point, returns true if the
     *     point is found.
     * @returns The first point that satisfies the predicate, or undefined if no
     *     point is found.
     */
    find(predicate: (point: OverlayPoint<T>) => boolean): OverlayPoint<T> | undefined;
    /**
     * Finds all slices that are contained within the given range. A slice is
     * considered contained if its start and end points are within the range,
     * inclusive (uses {@link Range#contains} method to check containment).
     *
     * @param range The range to search for contained slices.
     * @returns A set of slices that are contained within the given range.
     */
    findContained(range: Range<T>): Set<Slice<T>>;
    /**
     * Finds all slices that overlap with the given range. A slice is considered
     * overlapping if its start or end point is within the range, inclusive
     * (uses {@link Range#containsPoint} method to check overlap).
     *
     * @param range The range to search for overlapping slices.
     * @returns A set of slices that overlap with the given range.
     */
    findOverlapping(range: Range<T>): Set<Slice<T>>;
    /**
     * Returns `true` if the current character is a marker sentinel.
     *
     * @param id ID of the point to check.
     * @returns Whether the point is a marker point.
     */
    isMarker(id: ITimestampStruct): boolean;
    hash: number;
    refresh(slicesOnly?: boolean): number;
    readonly slices: Map<Slice<T>, [start: OverlayPoint<T>, end: OverlayPoint<T>]>;
    private refreshSlices;
    private insSlice;
    private insMarker;
    private delSlice;
    /**
     * Retrieve an existing {@link OverlayPoint} or create a new one, inserted
     * in the tree, sorted by spatial dimension.
     */
    private upsertPoint;
    /**
     * Inserts a point into the tree, sorted by spatial dimension.
     * @param point Point to insert.
     * @returns Returns the existing point if it was already in the tree.
     */
    private insPoint;
    private delPoint;
    leadingTextHash: number;
    protected refreshTextSlices(stateTotal: number): number;
    toString(tab?: string): string;
}
