"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PersistedSlice = void 0;
const hasOwnProperty_1 = require("@jsonjoy.com/util/lib/hasOwnProperty");
const Point_1 = require("../rga/Point");
const Range_1 = require("../rga/Range");
const hash_1 = require("../../../json-crdt/hash");
const printTree_1 = require("tree-dump/lib/printTree");
const constants_1 = require("./constants");
const clock_1 = require("../../../json-crdt-patch/clock");
const json_pretty_1 = require("../../../json-pretty");
const util_1 = require("./util");
const json_crdt_patch_1 = require("../../../json-crdt-patch");
/**
 * A persisted slice is a slice that is stored in a {@link Model}. It is used for
 * rich-text formatting and annotations.
 *
 * @todo Maybe rename to "saved", "stored", "mutable".
 */
class PersistedSlice extends Range_1.Range {
    static deserialize(model, txt, chunk, tuple) {
        const header = +tuple.get(0).view();
        const id1 = tuple.get(1).view();
        const id2 = (tuple.get(2).view() || id1);
        const type = tuple.get(3).view();
        if (typeof header !== 'number')
            throw new Error('INVALID_HEADER');
        if (!(id1 instanceof clock_1.Timestamp))
            throw new Error('INVALID_ID');
        if (!(id2 instanceof clock_1.Timestamp))
            throw new Error('INVALID_ID');
        (0, util_1.validateType)(type);
        const anchor1 = (header & 1 /* SliceHeaderMask.X1Anchor */) >>> 0 /* SliceHeaderShift.X1Anchor */;
        const anchor2 = (header & 2 /* SliceHeaderMask.X2Anchor */) >>> 1 /* SliceHeaderShift.X2Anchor */;
        const behavior = (header & 28 /* SliceHeaderMask.Behavior */) >>> 2 /* SliceHeaderShift.Behavior */;
        const rga = txt.str;
        const p1 = new Point_1.Point(rga, id1, anchor1);
        const p2 = new Point_1.Point(rga, id2, anchor2);
        const slice = new PersistedSlice(model, txt, chunk, tuple, behavior, type, p1, p2);
        return slice;
    }
    constructor(
    /** The `Model` where the slice is stored. */
    model, 
    /** The Peritext context. */
    txt, 
    /** The `arr` chunk of `arr` where the slice is stored. */
    chunk, 
    /** The `vec` node which stores the serialized contents of this slice. */
    tuple, behavior, type, start, end) {
        super(txt.str, start, end);
        this.model = model;
        this.txt = txt;
        this.chunk = chunk;
        this.tuple = tuple;
        this.start = start;
        this.end = end;
        // ----------------------------------------------------------------- Stateful
        this.hash = 0;
        this.rga = txt.str;
        this.id = chunk.id;
        this.behavior = behavior;
        this.type = type;
    }
    isSplit() {
        return this.behavior === 0 /* SliceBehavior.Marker */;
    }
    tupleApi() {
        return this.model.api.wrap(this.tuple);
    }
    // ---------------------------------------------------------------- mutations
    set(start, end = start) {
        super.set(start, end);
        this.update({ range: this });
    }
    /**
     * Expand range left and right to contain all invisible space: (1) tombstones,
     * (2) anchors of non-deleted adjacent chunks.
     */
    expand() {
        super.expand();
        this.update({ range: this });
    }
    update(params) {
        let updateHeader = false;
        const changes = [];
        if (params.behavior !== undefined) {
            this.behavior = params.behavior;
            updateHeader = true;
        }
        if (params.range) {
            const range = params.range;
            updateHeader = true;
            changes.push([1 /* SliceTupleIndex.X1 */, json_crdt_patch_1.s.con(range.start.id)], [2 /* SliceTupleIndex.X2 */, json_crdt_patch_1.s.con(range.end.id)]);
            this.start = range.start;
            this.end = range.start === range.end ? range.end.clone() : range.end;
        }
        if (params.type !== undefined) {
            this.type = params.type;
            changes.push([3 /* SliceTupleIndex.Type */, json_crdt_patch_1.s.con(this.type)]);
        }
        if ((0, hasOwnProperty_1.hasOwnProperty)(params, 'data'))
            changes.push([4 /* SliceTupleIndex.Data */, params.data]);
        if (updateHeader) {
            const header = (this.behavior << 2 /* SliceHeaderShift.Behavior */) +
                (this.start.anchor << 0 /* SliceHeaderShift.X1Anchor */) +
                (this.end.anchor << 1 /* SliceHeaderShift.X2Anchor */);
            changes.push([0 /* SliceTupleIndex.Header */, json_crdt_patch_1.s.con(header)]);
        }
        this.tupleApi().set(changes);
    }
    data() {
        return this.tuple.get(4 /* SliceTupleIndex.Data */)?.view();
    }
    dataNode() {
        const node = this.tuple.get(4 /* SliceTupleIndex.Data */);
        return node && this.model.api.wrap(node);
    }
    isDel() {
        return this.chunk.del;
    }
    refresh() {
        let state = 5381 /* CONST.START_STATE */;
        state = (0, hash_1.updateNode)(state, this.tuple);
        const changed = state !== this.hash;
        this.hash = state;
        if (changed) {
            const tuple = this.tuple;
            const slice = PersistedSlice.deserialize(this.model, this.txt, this.chunk, tuple);
            this.behavior = slice.behavior;
            this.type = slice.type;
            this.start = slice.start;
            this.end = slice.end;
        }
        return this.hash;
    }
    // ---------------------------------------------------------------- Printable
    toStringName() {
        const data = this.data();
        const dataFormatted = data ? (0, json_pretty_1.prettyOneLine)(data) : '∅';
        const dataLengthBreakpoint = 32;
        const header = `${this.constructor.name} ${super.toString('', true)}, ${constants_1.SliceBehaviorName[this.behavior]}, ${JSON.stringify(this.type)}${dataFormatted.length < dataLengthBreakpoint ? `, ${dataFormatted}` : ''}`;
        return header;
    }
    toString(tab = '') {
        const data = this.data();
        const dataFormatted = data ? (0, json_pretty_1.prettyOneLine)(data) : '';
        const dataLengthBreakpoint = 32;
        return (this.toStringName() +
            (0, printTree_1.printTree)(tab, [dataFormatted.length < dataLengthBreakpoint ? null : (tab) => dataFormatted]));
    }
}
exports.PersistedSlice = PersistedSlice;
