"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Slices = void 0;
const AvlMap_1 = require("sonic-forest/lib/avl/AvlMap");
const printTree_1 = require("tree-dump/lib/printTree");
const PersistedSlice_1 = require("./PersistedSlice");
const clock_1 = require("../../../json-crdt-patch/clock");
const hash_1 = require("../../../json-crdt/hash");
const json_hash_1 = require("../../../json-hash");
const MarkerSlice_1 = require("./MarkerSlice");
const nodes_1 = require("../../../json-crdt/nodes");
class Slices {
    constructor(
    /** The text RGA. */
    txt, 
    /** The `arr` node, used as a set, where slices are stored. */
    set) {
        this.txt = txt;
        this.set = set;
        this.list = new AvlMap_1.AvlMap(clock_1.compare);
        // ----------------------------------------------------------------- Stateful
        this._topologyHash = 0;
        this.hash = 0;
        this.rga = txt.str;
    }
    ins(range, behavior, type, data, Klass = behavior === 0 /* SliceBehavior.Marker */ ? MarkerSlice_1.MarkerSlice : PersistedSlice_1.PersistedSlice) {
        const slicesModel = this.set.doc;
        const set = this.set;
        const api = slicesModel.api;
        const builder = api.builder;
        const tupleId = builder.vec();
        const start = range.start.clone();
        const end = range.end.clone();
        const header = (behavior << 2 /* SliceHeaderShift.Behavior */) +
            (start.anchor << 0 /* SliceHeaderShift.X1Anchor */) +
            (end.anchor << 1 /* SliceHeaderShift.X2Anchor */);
        const headerId = builder.const(header);
        const x1Id = builder.const(start.id);
        const x2Id = builder.const((0, clock_1.compare)(start.id, end.id) === 0 ? 0 : end.id);
        const subtypeId = builder.const(type);
        const tupleKeysUpdate = [
            [0 /* SliceTupleIndex.Header */, headerId],
            [1 /* SliceTupleIndex.X1 */, x1Id],
            [2 /* SliceTupleIndex.X2 */, x2Id],
            [3 /* SliceTupleIndex.Type */, subtypeId],
        ];
        if (data !== undefined)
            tupleKeysUpdate.push([4 /* SliceTupleIndex.Data */, builder.json(data)]);
        builder.insVec(tupleId, tupleKeysUpdate);
        const chunkId = builder.insArr(set.id, set.id, [tupleId]);
        // TODO: Consider using `s` schema here.
        api.apply();
        const tuple = slicesModel.index.get(tupleId);
        const chunk = set.findById(chunkId);
        // TODO: Need to check if split slice text was deleted
        const slice = new Klass(slicesModel, this.txt, chunk, tuple, behavior, type, start, end);
        this.list.set(chunk.id, slice);
        return slice;
    }
    insMarker(range, type, data) {
        return this.ins(range, 0 /* SliceBehavior.Marker */, type, data);
    }
    insMarkerAfter(after, type, data, separator = "\n" /* Chars.BlockSplitSentinel */) {
        // TODO: test condition when cursors is at absolute or relative starts
        const txt = this.txt;
        const api = txt.model.api;
        const builder = api.builder;
        /**
         * We skip one clock cycle to prevent Block-wise RGA from merging adjacent
         * characters. We want the marker chunk to always be its own distinct chunk.
         */
        builder.nop(1);
        // TODO: Handle case when marker is inserted at the abs start, prevent abs start/end inserts.
        const textId = builder.insStr(txt.str.id, after, separator);
        api.apply();
        const point = txt.point(textId, 0 /* Anchor.Before */);
        const range = txt.range(point, point.clone());
        return this.insMarker(range, type, data);
    }
    insStack(range, type, data) {
        return this.ins(range, 1 /* SliceBehavior.Stack */, type, data);
    }
    insOverwrite(range, type, data) {
        return this.ins(range, 2 /* SliceBehavior.Overwrite */, type, data);
    }
    insErase(range, type, data) {
        return this.ins(range, 3 /* SliceBehavior.Erase */, type, data);
    }
    unpack(chunk) {
        const txt = this.txt;
        const model = this.set.doc;
        const tupleId = chunk.data ? chunk.data[0] : undefined;
        if (!tupleId)
            throw new Error('SLICE_NOT_FOUND');
        const tuple = model.index.get(tupleId);
        if (!(tuple instanceof nodes_1.VecNode))
            throw new Error('NOT_TUPLE');
        let slice = PersistedSlice_1.PersistedSlice.deserialize(model, txt, chunk, tuple);
        if (slice.isSplit())
            slice = new MarkerSlice_1.MarkerSlice(model, txt, chunk, tuple, slice.behavior, slice.type, slice.start, slice.end);
        return slice;
    }
    get(id) {
        return this.list.get(id);
    }
    del(id) {
        this.list.del(id);
        const set = this.set;
        const api = set.doc.api;
        // TODO: Is it worth checking if the slice is already deleted?
        api.builder.del(set.id, [(0, clock_1.tss)(id.sid, id.time, 1)]);
        api.apply();
    }
    delSlices(slices) {
        const api = this.set.doc.api;
        const spans = [];
        const length = slices.length;
        for (let i = 0; i < length; i++) {
            const slice = slices[i];
            if (slice instanceof PersistedSlice_1.PersistedSlice) {
                const id = slice.id;
                spans.push(new clock_1.Timespan(id.sid, id.time, 1));
            }
        }
        api.builder.del(this.set.id, spans);
        api.apply();
    }
    size() {
        return this.list._size;
    }
    iterator0() {
        const iterator = this.list.iterator0();
        return () => iterator()?.v;
    }
    forEach(callback) {
        this.list.forEach((node) => callback(node.v));
    }
    refresh() {
        const topologyHash = (0, hash_1.updateRga)(5381 /* CONST.START_STATE */, this.set);
        if (topologyHash !== this._topologyHash) {
            this._topologyHash = topologyHash;
            let chunk;
            for (const iterator = this.set.iterator(); (chunk = iterator());) {
                const item = this.list.get(chunk.id);
                if (chunk.del) {
                    if (item)
                        this.list.del(chunk.id);
                }
                else {
                    if (!item)
                        this.list.set(chunk.id, this.unpack(chunk));
                }
            }
        }
        let hash = topologyHash;
        this.list.forEach(({ v: item }) => {
            item.refresh();
            hash = (0, json_hash_1.updateNum)(hash, item.hash);
        });
        return (this.hash = hash);
    }
    // ---------------------------------------------------------------- Printable
    toString(tab = '') {
        return (this.constructor.name +
            (0, printTree_1.printTree)(tab, [...this.list.entries()].map(({ v }) => (tab) => v.toString(tab))));
    }
}
exports.Slices = Slices;
