import * as operations from './operations';
import { ITimestampStruct } from './clock';
import type { Printable } from 'tree-dump/lib/types';
/**
 * A union type of all possible JSON CRDT patch operations.
 */
export type JsonCrdtPatchOperation = operations.NewConOp | operations.NewValOp | operations.NewVecOp | operations.NewObjOp | operations.NewStrOp | operations.NewBinOp | operations.NewArrOp | operations.InsValOp | operations.InsObjOp | operations.InsVecOp | operations.InsStrOp | operations.InsBinOp | operations.InsArrOp | operations.DelOp | operations.NopOp;
/**
 * Represents a JSON CRDT patch.
 *
 * Normally, you would create a new patch using the {@link PatchBuilder} class.
 *
 * ```ts
 * import {Patch, PatchBuilder, LogicalClock} from 'json-joy/lib/json-crdt-patch';
 *
 * const clock = new LogicalClock(3, 100);
 * const builder = new PatchBuilder(clock);
 * const patch = builder.flush();
 * ```
 *
 * Save patch to a binary representation:
 *
 * ```ts
 * const binary = patch.toBinary();
 * ```
 *
 * Load patch from a binary representation:
 *
 * ```ts
 * const patch = Patch.fromBinary(binary);
 * ```
 *
 * @category Patch
 */
export declare class Patch implements Printable {
    /**
     * Un-marshals a JSON CRDT patch from a binary representation.
     */
    static fromBinary(data: Uint8Array): Patch;
    /**
     * A list of operations in the patch.
     */
    readonly ops: JsonCrdtPatchOperation[];
    /**
     * Arbitrary metadata associated with the patch, which is not used by the
     * library.
     */
    meta: unknown;
    /**
     * Returns the patch ID, which is equal to the ID of the first operation
     * in the patch.
     *
     * @returns The ID of the first operation in the patch.
     */
    getId(): ITimestampStruct | undefined;
    /**
     * Returns the total time span of the patch, which is the sum of all
     * operation spans.
     *
     * @returns The length of the patch.
     */
    span(): number;
    /**
     * Returns the expected time of the next inserted operation.
     */
    nextTime(): number;
    /**
     * Creates a new patch where all timestamps are transformed using the
     * provided function.
     *
     * @param ts Timestamp transformation function.
     * @returns A new patch with transformed timestamps.
     */
    rewriteTime(ts: (id: ITimestampStruct) => ITimestampStruct): Patch;
    /**
     * The .rebase() operation is meant to work only with patch that use
     * the server clock. When receiving a patch from a client, the starting
     * ID of the patch can be out of sync with the server clock. For example,
     * if some other user has in the meantime pushed operations to the server.
     *
     * The .rebase() operation returns a new `Patch` with the IDs recalculated
     * such that the first operation has ID of the patch is equal to the
     * actual server time tip.
     *
     * @param serverTime Real server time tip (ID of the next expected operation).
     */
    rebase(serverTime: number, transformHorizon: number): Patch;
    /**
     * Creates a deep clone of the patch.
     *
     * @returns A deep clone of the patch.
     */
    clone(): Patch;
    /**
     * Marshals the patch into a binary representation.
     *
     * @returns A binary representation of the patch.
     */
    toBinary(): Uint8Array;
    /**
     * Returns a textual human-readable representation of the patch. This can be
     * used for debugging purposes.
     *
     * @param tab Start string for each line.
     * @returns Text representation of the patch.
     */
    toString(tab?: string): string;
}
