import { AvlMap } from 'sonic-forest/lib/avl/AvlMap';
import { ITimestampStruct, Patch } from '../../json-crdt-patch';
import { Model } from '../model';
import type { Printable } from 'tree-dump/lib/types';
import type { JsonNode } from '../nodes/types';
/**
 * The `Log` represents a history of patches applied to a JSON CRDT model. It
 * consists of: (1) a starting {@link Model} instance, (2) a list of {@link Patch} instances,
 * that can be applied to the starting model to reach the current state of the
 * document, and (3) the current state of the document, the `end` {@link Model}.
 *
 * The log can be used to replay the history of patches to any point in time,
 * from the "start" to the "end" of the log, and return the resulting {@link Model}
 * state.
 */
export declare class Log<N extends JsonNode = JsonNode<any>> implements Printable {
    /**
     * Creates a `PatchLog` instance from a newly JSON CRDT model. Checks if
     * the model API buffer has any initial operations applied, if yes, it
     * uses them to create the initial state of the log.
     *
     * @param model A new JSON CRDT model, just created with
     *              `Model.withLogicalClock()` or `Model.withServerClock()`.
     * @returns A new `PatchLog` instance.
     */
    static fromNewModel<N extends JsonNode = JsonNode<any>>(model: Model<N>): Log<N>;
    /**
     * Model factory function that creates a new JSON CRDT model instance, which
     * is used as the starting point of the log. It is called every time a new
     * model is needed to replay the log.
     *
     * @readonly Internally this function may be updated, but externally it is
     *           read-only.
     */
    start: () => Model<N>;
    /**
     * The end of the log, the current state of the document. It is the model
     * instance that is used to apply new patches to the log.
     *
     * @readonly
     */
    readonly end: Model<N>;
    /**
     * The collection of patches which are applied to the `start()` model to reach
     * the `end` model. The patches in the log, stored in an AVL tree for
     * efficient replaying. The patches are sorted by their logical timestamps
     * and applied in causal order.
     *
     * @readonly
     */
    readonly patches: AvlMap<ITimestampStruct, Patch>;
    private __onPatch;
    private __onFlush;
    constructor(start: () => Model<N>);
    /**
     * Call this method to destroy the `PatchLog` instance. It unsubscribes patch
     * and flush listeners from the `end` model and clears the patch log.
     */
    destroy(): void;
    /**
     * Creates a new model instance using the `start()` factory function and
     * replays all patches in the log to reach the current state of the document.
     *
     * @returns A new model instance with all patches replayed.
     */
    replayToEnd(): Model<N>;
    /**
     * Replays the patch log until a specified timestamp, including the patch
     * at the given timestamp. The model returned is a new instance of `start()`
     * with patches replayed up to the given timestamp.
     *
     * @param ts Timestamp ID of the patch to replay to.
     * @returns A new model instance with patches replayed up to the given timestamp.
     */
    replayTo(ts: ITimestampStruct): Model<N>;
    /**
     * Advance the start of the log to a specified timestamp, excluding the patch
     * at the given timestamp. This method removes all patches from the log that
     * are older than the given timestamp and updates the `start()` factory
     * function to replay the log from the new start.
     *
     * @param ts Timestamp ID of the patch to advance to.
     */
    advanceTo(ts: ITimestampStruct): void;
    toString(tab?: string): string;
}
