"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Log = void 0;
const AvlMap_1 = require("sonic-forest/lib/avl/AvlMap");
const util_1 = require("sonic-forest/lib/util");
const printTree_1 = require("tree-dump/lib/printTree");
const json_crdt_patch_1 = require("../../json-crdt-patch");
const model_1 = require("../model");
/**
 * The `Log` represents a history of patches applied to a JSON CRDT model. It
 * consists of: (1) a starting {@link Model} instance, (2) a list of {@link Patch} instances,
 * that can be applied to the starting model to reach the current state of the
 * document, and (3) the current state of the document, the `end` {@link Model}.
 *
 * The log can be used to replay the history of patches to any point in time,
 * from the "start" to the "end" of the log, and return the resulting {@link Model}
 * state.
 */
class Log {
    /**
     * Creates a `PatchLog` instance from a newly JSON CRDT model. Checks if
     * the model API buffer has any initial operations applied, if yes, it
     * uses them to create the initial state of the log.
     *
     * @param model A new JSON CRDT model, just created with
     *              `Model.withLogicalClock()` or `Model.withServerClock()`.
     * @returns A new `PatchLog` instance.
     */
    static fromNewModel(model) {
        const clock = model.clock.clone();
        const log = new Log(() => new model_1.Model(clock));
        const api = model.api;
        if (api.builder.patch.ops.length)
            log.end.applyPatch(api.flush());
        return log;
    }
    constructor(start) {
        /**
         * The collection of patches which are applied to the `start()` model to reach
         * the `end` model. The patches in the log, stored in an AVL tree for
         * efficient replaying. The patches are sorted by their logical timestamps
         * and applied in causal order.
         *
         * @readonly
         */
        this.patches = new AvlMap_1.AvlMap(json_crdt_patch_1.compare);
        this.start = start;
        const end = (this.end = start());
        const onPatch = (patch) => {
            const id = patch.getId();
            if (!id)
                return;
            this.patches.set(id, patch);
        };
        const api = end.api;
        this.__onPatch = api.onPatch.listen(onPatch);
        this.__onFlush = api.onFlush.listen(onPatch);
    }
    /**
     * Call this method to destroy the `PatchLog` instance. It unsubscribes patch
     * and flush listeners from the `end` model and clears the patch log.
     */
    destroy() {
        this.__onPatch();
        this.__onFlush();
        this.patches.clear();
    }
    /**
     * Creates a new model instance using the `start()` factory function and
     * replays all patches in the log to reach the current state of the document.
     *
     * @returns A new model instance with all patches replayed.
     */
    replayToEnd() {
        const clone = this.start().clone();
        for (let node = (0, util_1.first)(this.patches.root); node; node = (0, util_1.next)(node))
            clone.applyPatch(node.v);
        return clone;
    }
    /**
     * Replays the patch log until a specified timestamp, including the patch
     * at the given timestamp. The model returned is a new instance of `start()`
     * with patches replayed up to the given timestamp.
     *
     * @param ts Timestamp ID of the patch to replay to.
     * @returns A new model instance with patches replayed up to the given timestamp.
     */
    replayTo(ts) {
        const clone = this.start().clone();
        for (let node = (0, util_1.first)(this.patches.root); node && (0, json_crdt_patch_1.compare)(ts, node.k) >= 0; node = (0, util_1.next)(node))
            clone.applyPatch(node.v);
        return clone;
    }
    /**
     * Advance the start of the log to a specified timestamp, excluding the patch
     * at the given timestamp. This method removes all patches from the log that
     * are older than the given timestamp and updates the `start()` factory
     * function to replay the log from the new start.
     *
     * @param ts Timestamp ID of the patch to advance to.
     */
    advanceTo(ts) {
        const newStartPatches = [];
        let node = (0, util_1.first)(this.patches.root);
        for (; node && (0, json_crdt_patch_1.compare)(ts, node.k) >= 0; node = (0, util_1.next)(node))
            newStartPatches.push(node.v);
        for (const patch of newStartPatches)
            this.patches.del(patch.getId());
        const oldStart = this.start;
        this.start = () => {
            const model = oldStart();
            for (const patch of newStartPatches)
                model.applyPatch(patch);
            return model;
        };
    }
    // ---------------------------------------------------------------- Printable
    toString(tab) {
        const patches = [];
        this.patches.forEach(({ v }) => patches.push(v));
        return (`log` +
            (0, printTree_1.printTree)(tab, [
                (tab) => `start` + (0, printTree_1.printTree)(tab, [(tab) => this.start().toString(tab)]),
                () => '',
                (tab) => 'history' +
                    (0, printTree_1.printTree)(tab, patches.map((patch, i) => (tab) => `${i}: ${patch.toString(tab)}`)),
                () => '',
                (tab) => `end` + (0, printTree_1.printTree)(tab, [(tab) => this.end.toString(tab)]),
            ]));
    }
}
exports.Log = Log;
