import { Codegen } from '@jsonjoy.com/util/lib/codegen';
import { ValidationError } from '../../constants';
import type { TypeSystem } from '../../system';
import type { Type } from '../../type';
import type { ValidationPath } from './types';
export interface ValidatorCodegenContextOptions {
    /** Type for which to generate the validator. */
    type: Type;
    /** Type system to use for alias and validator resolution. */
    system?: TypeSystem;
    /**
     * Specifies how errors should be reported. The validator always returns a truthy
     * value on error, and falsy value on success. Depending on the value of this
     * option, the validator will either return boolean, string, or object on error.
     *
     * - `"boolean"`: The validator will return `true` on error, and `false` on success.
     * - `"string"`: The validator will return a string on error, and empty string `""`
     *   on success. The error string contains error code and path where error happened
     *   serialized as JSON.
     * - `"object"`: The validator will return an object on error, and `null` on success. The
     *   error object contains error code and path where error happened as well as human readable
     *   description of the error.
     *
     * Use `"boolean"` for best performance.
     */
    errors: 'boolean' | 'string' | 'object';
    /**
     * When an object type does not have "extraFields" set to true, the validator
     * will check that there are not excess fields besides those explicitly
     * defined. This settings removes this check.
     *
     * It may be useful when validating incoming data in RPC request as extra fields
     * would not hurt, but removing this check may improve performance. In one
     * micro-benchmark, this setting improves performance 5x. See json-type/validator.js benchmark.
     */
    skipObjectExtraFieldsCheck?: boolean;
    /**
     * In unsafe mode the validator will skip some checks which may result in
     * error being thrown. When running validators in unsafe mode, it is assumed
     * that the code is wrapped in a try-catch block. Micro-benchmarks DO NOT show
     * that this setting improves performance much.
     */
    unsafeMode?: boolean;
}
export declare class ValidatorCodegenContext {
    readonly options: ValidatorCodegenContextOptions;
    readonly codegen: Codegen;
    constructor(options: ValidatorCodegenContextOptions);
    js(js: string): void;
    /** Generates an error message. The type of message form is dictated by `options.errors` setting. */
    err(code: ValidationError, path: ValidationPath, opts?: {
        refId?: string;
        refError?: string;
        validator?: string;
    }): string;
    emitCustomValidators(node: Type, path: ValidationPath, r: string): void;
    private validatorCache;
    protected linkValidator(name: string): string;
    compile(): (...deps: unknown[]) => unknown;
}
